#
# Script Helper Function Library
# Warning: The integrity of this file is verified by Ciphire Mail.
#          Do not modify!
# Copyright (c) 2003-2005 Ciphire Labs
#

lmkdir()
{
  path=""
  dirs=`echo $1 | sed -e 's%/% %g'`
  mode=$2

  set -- ${dirs}

  for d in ${dirs}
  do
    path="${path}/$d"
    if [ ! -d "${path}" ]
    then
      mkdir ${path}
      if [ $? -ne 0 ]
      then
        echo "Failed to create directory: ${path}"
        exit 1
      fi
      if [ "x${mode}" != x ]
      then
        chmod ${mode} ${path}
      fi
    fi
  done
}
 
lcopy()
{
  from=$1
  to=$2
  dmode=$3
  fmode=$4

  test -d ${to} || lmkdir ${to} ${dmode}
  (cd ${from} && tar -cf - .) | (cd ${to} && tar -xf -)

  if [ "x${fmode}" != x ]
  then
    find ${to} -type f -print | xargs chmod ${fmode}
  fi
  if [ "x${dmode}" != x ]
  then
    find ${to} -type d -print | xargs chmod ${dmode}
  fi
}
 
checkyesno()
{
  local e
  while true
  do
    read YN
    case "$YN" in
      y|Y|yes|Yes|YES)
        e=0
        break
      ;;
      n|N|no|No|NO)
        e=1
        break
      ;;
      *)
        echo Please enter "y" or "n". 
      ;;
    esac
  done
  if [ "x$1" = "x" ]
  then
    if [ $e = 1 ]
    then
      exit 1
    fi
  fi
  return $e
}

add_line_to_file()
{
  OIFS="$IFS"
  IFS='
'
  ADDED=
  from=$1
  to=$2
  addline=$3

  touch $to
  for line in `cat $from`
  do
    if [ ! $ADDED ]
    then
      if [ "x`echo \"$line\" | grep '^#'`" = "x" ]
      then
        echo "$addline" >> $to
        ADDED=1
      fi
    fi
    echo $line >> $to
  done
  IFS=$OIFS
}

remove_line_from_file()
{
  OIFS="$IFS"
  IFS='
'
  REMOVED=
  from=$1
  to=$2
  matchstring=$3

  touch $to
  for line in `cat $from`
  do
    if [ ! $REMOVED ]
    then
      if [ "x`echo \"$line\" | grep \"$matchstring\"`" != "x" ]
      then
        REMOVED=1
        line=""
      fi
    fi
    echo $line >> $to
  done
  IFS=$OIFS
}

file_has_redir()
{
  if [ "`grep ciphire-redir $1`" ]
  then
    return 0
  else
    return 1
  fi
}

file_has_ciphire()
{
  if [ "`grep ciphire $1`" ]
  then
    return 0
  else
    return 1
  fi
}

file_has_tryexec()
{
  if [ "`grep -i ^tryexec $1`" ]
  then
    return 0
  else
    return 1
  fi
}

add_redir_to_file()
{
  add_file=$1
  add_cidir=$2
  if [ -e $add_file ]
  then
    name=`basename $add_file`
    tmpfile=/tmp/.$name.ciphire.$$
    
    if ! file_has_redir $add_file
    then
      add_line_to_file $add_file $tmpfile ". $add_cidir/setupdata/ciphire-redir.startup"
      if [ -e $tmpfile ]
      then
        cp $tmpfile $add_file
        rm -f $tmpfile
      else
        return 1
      fi
    fi
    return 0
  else
    return 1
  fi
}

remove_redir_from_file()
{
  rem_file=$1
  if [ -e $rem_file ]
  then
    name=`basename $rem_file`
    tmpfile=/tmp/.$name.ciphire.r$$
    
    if file_has_ciphire $rem_file
    then
      remove_line_from_file $rem_file $tmpfile "ciphire-redir.startup"
      if [ -e $tmpfile ]
      then
        cp $tmpfile $rem_file
        rm -f $tmpfile
      else
        return 1
      fi
    fi
    return 0
  else
    return 1
  fi
}

locate_desktop_managers()
{
  DESKTOPFILE_DIRS=("/usr/share/xsessions"
    "/etc/X11/dm/Sessions/"
    "/opt/kde3/share/apps/kdm/sessions"
    "/usr/share/apps/kdm/sessions"
    "/opt/gnome/share/xsessions"
    "/usr/share/gnome/wm-properties")
  xexecutables=()
  XPOINTER=0
  for DIR in ${DESKTOPFILE_DIRS[@]} 
  do
    if [ -d $DIR ]
    then
      for dfile  in `find $DIR -type f -name '*.desktop'`
      do
        exec_file="`grep -i ^exec $dfile | sed -r 's|^[eE][xX][eE][cC][]*=[ ]*([^ ]*).*|\1|'`"
        exec_file=`which $exec_file 2>/dev/null`
        if [ "$exec_file" ]
        then
          if [ -e $exec_file -a -z "`echo ${xexecutables[@]} | grep $exec_file`" ]
          then
            xexecutables[$XPOINTER]=$exec_file
            let "XPOINTER = $XPOINTER + 1"
          fi
        else
          tryexec_file="`grep -i ^tryexec $dfile | sed -r 's|^[tT][rR][yY][eE][xX][eE][cC][]*=[ ]*([^ ]*).*|\1|'`"
          tryexec_file=`which $tryexec_file 2>/dev/null`
          if [ "$tryexec_file" ]
          then
            if [ -e $tryexec_file -a -z "`echo ${xexecutables[@]} | grep $tryexec_file`" ]
            then
              xexecutables[$XPOINTER]=$tryexec_file
              let "XPOINTER = $XPOINTER + 1"
            fi
          fi
        fi
      done
    fi
  done
  if [ "x$XPOINTER" = "x0" ]
  then
    return 1
  else
    return 0
  fi
}

wrap_desktop_managers()
{
  XSESSION_WRAPPERS=
  if locate_desktop_managers
  then
    for xfile in ${xexecutables[@]}
    do
      if file_has_ciphire $xfile
      then
        writelog "$xfile seems to be a wrapper already. Skipping ..."
      else
        mv $xfile $xfile.ciphire.orig
        echo "#!/bin/sh"                                                > $xfile
        echo "# Generated by Ciphire Mail Unix Installer ($DATE)"                   >> $xfile
        echo "if [ -e $CITARGET/ciphire-ctl ]"                           >> $xfile
        echo "then"                                                    >> $xfile
        echo "  $CITARGET/ciphire-ctl xstart $xfile.ciphire.orig \$@" >> $xfile
        echo "else"                                                    >> $xfile
        echo "  exec $xfile.ciphire.orig \$@"                       >> $xfile
        echo "fi"                                                      >> $xfile
        chmod 755 $xfile
        writelog "Wrapped $xfile"
      fi
      if [ "$XSESSION_WRAPPERS" ]
      then
        XSESSION_WRAPPERS="$XSESSION_WRAPPERS:$xfile"
      else
        XSESSION_WRAPPERS="$xfile"
      fi
    done
    return 0
  else
    return 1
  fi
}

unwrap_desktop_managers()
{
  IFS0=$IFS
  IFS=":"
  for wfile in $XSESSION_WRAPPERS
  do
    if [ -e $wfile.ciphire.orig ]
    then
      if file_has_ciphire $wfile
      then
        mv $wfile.ciphire.orig $wfile
        writelog "Unwrapped $wfile"
      else
        writelog "Wrapper $wfile has been replaced by someone else. Skipping ..."
      fi
    else
      writelog "Warning: $wfile.ciphire.orig not found. Can't restore original file: $wfile"
    fi
  done
  IFS=$IFS0
}

show_desktop_managers()
{
  if [ $XSESSION_WRAPPERS ]
  then
    IFS0=$IFS
    IFS=":"
    for xfile in $XSESSION_WRAPPERS
    do
      echo -en "`basename $xfile` "
    done
    IFS=$IFS0
  else
    echo -en "(none)"
  fi
}

writelog()
{
  LOGFILE=/tmp/install-ciphire.log
  if [ ! -e $LOGFILE ]
  then
    touch $LOGFILE
  fi
  if [ -f $LOGFILE -a -w $LOGFILE ]
  then
    echo "$DATE $CN $1" >> $LOGFILE
  fi
}

config_dir_exists()
{
  if [ ! -e "$HOME/.ciphire" ]
  then
    echo "ERROR: Ciphire configuration directory not found!"
    echo "Maybe you need to run 'ciphire-setup' first?"
    exit 1
  fi
}

kill_transformer()
{
  CIPID="`cat $CIPIDFILE`"
  if [ "x$CIPID" != "x" ]
  then
    limit=0
    while [ $limit -lt 20 ] && kill -0 $CIPID >/dev/null 2>&1
    do
      kill $CIPID >/dev/null 2>&1
      sleep 1
      let "limit = $limit + 1"
    done
    if kill -0 $CIPID >/dev/null 2>&1
    then
      kill -9 $CIPID >/dev/null 2>&1
      sleep 1
      if kill -0 $CIPID >/dev/null 2>&1
      then
        return 1
      else
       return 0
      fi
    else
      return 0
    fi
  else
    return 0
  fi
}

kill_launcher()
{
  CILPID="`cat $CILPIDFILE`"
  if [ "x$CILPID" != "x" ]
  then
    limit=0
    while [ $limit -lt 5 ] && kill -0 $CILPID >/dev/null 2>&1
    do
      kill $CILPID >/dev/null 2>&1
      sleep 1
      let "limit = $limit + 1"
    done
    if kill -0 $CILPID >/dev/null 2>&1
    then
      kill -9 $CILPID >/dev/null 2>&1
      sleep 1
      if kill -0 $CLIPID >/dev/null 2>&1
      then
        return 1
      else
       return 0
      fi
    else
      return 0
    fi
  else
    return 0
  fi
}

transformer_is_running()
{
  if [ -f $CIPIDFILE ]
  then
    CIPID="`cat $CIPIDFILE`"
    if [ "x$CIPID" != "x" ] && kill -0 $CIPID >/dev/null 2>&1
    then
      return 0
    fi
  fi
  return 1
}

transformer_started()
{
  limit=0
  while [ $limit -lt 60 ] && ! transformer_is_running
  do
    sleep 1
    let "limit = $limit + 1"
  done
  if transformer_is_running
  then
    return 0
  else
    return 1
  fi
}

launcher_is_running()
{
  if [ -f $CILPIDFILE ]
  then
    CIPID="`cat $CILPIDFILE`"
    if [ "x$CIPID" != "x" ] && kill -0 $CIPID >/dev/null 2>&1
    then
      return 0
    fi
  fi
  return 1
}

redirector_is_active()
{
  if [ "x$LD_PRELOAD" != "x" ]
  then
    if [ "x`echo $LD_PRELOAD | grep ciphire-redir.so 2>/dev/null`" != "x" ]
    then
      return 0
    fi
  fi
  return 1
}

launcher_needs_restart()
{
  if [ "x`find $CIDIR/.sup.restart -cmin 10 2>/dev/null`" = "x" ]
  then
    return 1
  fi
  return 0
}

sup_processor_is_active()
{
  if [ -f $CIDIR/.sup.lock ]
  then
    return 0
  fi
  return 1
}

call_sup_processor()
{
  if [ "x$1" = "x" ]
  then
    SUPPRO_PARAM=" --uid $USERID --gid $GROUPID --vfolder binary-folder $CIDIR main --user-folder $CIUSERDIR"
  else
    SUPPRO_PARAM="$*"
  fi

  $CIDIR/ciphire-sup-processor $SUPPRO_PARAM >/dev/null 2>&1
  case $? in
    0) # ok (or ER-CSR sent, uninstall now)
      U_NOW=1
       return 0
      ;;
    4) # need root privileges
      if $CIDIR/ciphire-sup
      then
        return 0
      else
        return 1
      fi
      ;;
    20) # ER-CSR sent, uninstall later (current)
      U_EDS=1
      U_LATER=1
      U_UDU=1
      return 0
      ;;
    30) # ER-CSR sent, uninstall later (all)
      U_EDS=1
      U_LATER=1
      return 0
      ;;
    31) # ER-CSR sent, uninstall now
      U_EDS=1
      U_NOW=1
      return 0
      ;;
    22) # user pressed cancel
      return 1
      ;;
    *) # other error
      return 1
      ;;
  esac
}

install_desktop_file()
{
  fromdir=$1
  todir=$2
  cidir=$3
  if [ ! -e $todir ]
  then
   lmkdir $todir 755
  fi
  sed -e "s|@@cidir@@|$cidir|g" $fromdir/ciphire.desktop > $todir/ciphire.desktop
  chmod 644 $todir/ciphire.desktop
}

remove_desktop_file()
{
  todir=$1
  if [ -e "$todir/ciphire.desktop" ]
  then
   rm -f $todir/ciphire.desktop
  fi
}

have_display()
{
  POPUP="$CIDIR/ciphire-msg -s"
  TITLE="Ciphire Mail 0.11.29"
  if [ ! -e $CIDIR ]
  then
    echo "ERROR: ciphire-msg not found!"
    exit 1
  fi
  case $1 in
    1)
      return 0
      ;;
    0)
      return 1
      ;;
  esac
  if [ "$DISPLAY" ]
  then
    $POPUP '' >/dev/null 2>&1
    if [ "x$?" = "x154" ]
    then
      return 0
    else
      return 1
    fi
  else
    return 1
  fi
}

popup()
{
  POPUP="$CIDIR/ciphire-msg -s"
  TITLE="Ciphire Mail 0.11.29"
  MESSAGE="$2"
  if [ ! -e $CIDIR ]
  then
    echo "ERROR: ciphire-msg not found!"
    exit 1
  fi
  case $3 in
    qm)
      ICON="|1"
      break
      ;;
    warn)
      ICON="|3"
      break
      ;;
    error)
      ICON="|4"
      break
      ;;
    none)
      ICON="|0"
      break
      ;;
    *)
      ICON="|2"
      ;;
  esac
  case $1 in
    cont)
      BUTTONS="#Continue #Abort"
      BVAL0="return 0"
      BVAL1="exit 1"
      $POPUP "$ICON $BUTTONS \b$TITLE\n\n\p$MESSAGE" >/dev/null 2>&1
      PRESSED_BUTTON=$?
      break
      ;;
    yesno)
      BUTTONS="#Yes #No #Abort"
      BVAL0="return 0"
      BVAL1="return 1"
      BVAL2="exit 1"
      $POPUP "$ICON $BUTTONS \b$TITLE\n\n\p$MESSAGE" >/dev/null 2>&1
      PRESSED_BUTTON=$?
      break
      ;;
    input)
      BUTTONS="#Ok"
      BVAL0="return 0"
      DEFAULT_VALUE="$4"
      INPUT_VALUE=`$POPUP $POPUP_PARAM "$ICON \\$$DEFAULT_VALUE $BUTTONS \b$TITLE\n\n\p$MESSAGE" 2>/dev/null`
      if [ "x$INPUT_VALUE" = "x" ] 
      then
        INPUT_VALUE=$DEFAULT_VALUE
      fi
      PRESSED_BUTTON=$?
      break
      ;;
    error)
      BUTTONS="#Exit"
      BVAL0="exit 1"
      $POPUP "$ICON $BUTTONS \bERROR - $TITLE\n\n\p$MESSAGE" >/dev/null 2>&1
      PRESSED_BUTTON=$?
      break
      ;;
    license)
      BUTTONS="#Yes #No"
      BVAL0="return 0"
      BVAL1="exit 1"
      $POPUP "$ICON $BUTTONS \b$TITLE\n\n\p$MESSAGE\-\+files/docs/license/LICENSE.html \-$MESSAGE" >/dev/null 2>&1
      PRESSED_BUTTON=$?
      break
      ;;
    nop)
      BUTTONS="#Ok"
      BVAL0="return 0"
      $POPUP "$ICON $BUTTONS \b$TITLE\n\n\p$MESSAGE" >/dev/null 2>&1
      PRESSED_BUTTON=$?
      ;;
  esac
  case $PRESSED_BUTTON in
    0)
      $BVAL0
      ;;
    1)
      $BVAL1
      ;;
    2)
      $BVAL2
      ;;
  esac
}

echoask()
{
  MESSAGE="$2"
  DEFAULT_VALUE="$4"
  case $1 in
    cont)
      echo -en "$2Continue? (y/n) "
      checkyesno
      return 0
      ;;
    yesno)
      echo -en "$2Yes or No? (y/n) "
      checkyesno e
      if [ "$?" = "0" ]
      then
        return 0
      else
        return 1
      fi
      ;;
    input)
      echo -en "$MESSAGE[$DEFAULT_VALUE] "
      read INPUT_VALUE
      if [ "x$INPUT_VALUE" = "x" ] 
      then
        INPUT_VALUE=$DEFAULT_VALUE
      fi
      return 0
      ;;
    error)
      echo -en "ERROR: $MESSAGE"
      exit 1
      ;;
    license)
      more files/docs/license/LICENSE.txt
      echo -en "\n$2Yes or No? (yes/no) "
      checkyesno
      return 0
      ;;
    nop)
      echo -en "$MESSAGE"
      return 0
      ;;
  esac
}


set_popecho()
{
  if have_display $1
  then
    popecho()
    {
      popup "$1" "$2" "$3" "$4"
    }
    return 0
  else
    popecho()
    {
      echoask "$1" "$2" "$3" "$4"
    }
    return 1
  fi
}

write_distconfig()
{
  cidir=$1
  if [ ! "$DATE" ]
  then
    DATE=`date`
  fi
  echo "# Ciphire Mail Distribution Configuration ($DATE)" > $cidir/setupdata/ciphire.distconfig
  echo "XSESSION_WRAPPERS=$XSESSION_WRAPPERS"                >> $cidir/setupdata/ciphire.distconfig	
  echo "XSESSIONINITDIR_LOCATION=$XSESSIONINITDIR_LOCATION"  >> $cidir/setupdata/ciphire.distconfig
  echo "XSESSIONDIR_LOCATION=$XSESSIONDIR_LOCATION"          >> $cidir/setupdata/ciphire.distconfig
  echo "XSESSIONFILE_LOCATION=$XSESSIONFILE_LOCATION"        >> $cidir/setupdata/ciphire.distconfig
  echo "HAVE_SWITCHDESK=$HAVE_SWITCHDESK"                    >> $cidir/setupdata/ciphire.distconfig
  echo "HAVE_DXSM=$HAVE_DXSM"                                >> $cidir/setupdata/ciphire.distconfig
  echo "XSUPPORT=$XSUPPORT"                                  >> $cidir/setupdata/ciphire.distconfig
  echo "PRELOAD_VALUE=$PRELOAD_VALUE"                        >> $cidir/setupdata/ciphire.distconfig
  chmod 644 $cidir/setupdata/ciphire.distconfig
}
